package com.servicemall.right.micro;

import java.util.List;
import java.util.Map;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;

import com.servicemall.loyalty.data.Selected;
import com.servicemall.loyalty.service.SeletedService;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import team.bangbang.common.data.Pagination;
import team.bangbang.common.data.response.ResponseBase;
import team.bangbang.common.data.response.DataResponse;
import team.bangbang.common.log.OperationLog;
import team.bangbang.common.utility.LogicUtility;
import team.bangbang.spring.parameter.EntityParam;

import com.servicemall.right.data.Coupon;
import com.servicemall.right.service.CouponService;

/**
 * 优惠权益 - 微服务
 *
 * 对应Feign：com.servicemall.right.feign.CouponFeign
 *
 * @author 帮帮组
 * @version 1.0  2020-12-08
 */
@Api(tags = {"right"})
@ApiResponses(value={@ApiResponse(code=200, message="请求的服务不存在")})
@RestController
@CrossOrigin(allowCredentials="true", allowedHeaders="*", origins="*", maxAge=3600)
@RequestMapping("/microservice/right/coupon")
public final class CouponMicro {
	/**************************************************************************
	 * ！！除非设计、指导人员有特别说明，否则此处不得随意增加、修改、删除！！
	 * －－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－
	 *
	 *************************************************************************/
	/**
	 * 插入一条优惠权益
	 *
	 * @param coupon 插入的数据，不能为null
     * @param request HTTP请求
	 *
	 * @return 1：成功 其它：失败
	 */
	@ApiIgnore
	@PostMapping("/insert")
	public ResponseBase insert(@RequestBody Coupon coupon, HttpServletRequest request) {
		// 是否存在重复记录？
		if(exist(coupon)) {
			return ResponseBase.DATA_DUPLICATE;
		}

		int n = CouponService.insert(coupon);

		if(n != 1) {
			return ResponseBase.ERROR_LOGIC;
		}

        // 记录日志
        OperationLog log = new OperationLog();
        log.setType("新增优惠权益");
        log.setBizData(coupon);
        request.setAttribute("log", log);

		DataResponse<Coupon> result = new DataResponse<Coupon>();
		result.setData(coupon);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 删除优惠权益
	 *
	 * @param coupon 删除条件，必须包含关键字段
     * @param request HTTP请求
	 *
	 * @return 成功删除的记录数量
	 */
	@ApiIgnore
	@PostMapping("/delete")
	public ResponseBase delete(@RequestBody Coupon coupon, HttpServletRequest request) {

		if(coupon.getId() == null || coupon.getId() == 0L) {
			return ResponseBase.DATA_NOT_FOUND;
		}

        // 获取待删除的对象，用于日志记录
        coupon = CouponService.getObject(coupon, null);
        if(coupon == null) {
            return ResponseBase.DATA_NOT_FOUND;
        }

		// 限定条件
		Coupon where = new Coupon();
		where.setId(coupon.getId());

		int result = CouponService.delete(where, null);

		if(result != 1) {
			return ResponseBase.DATA_NOT_FOUND;
		}

        // 记录日志
        OperationLog log = new OperationLog();
        log.setType("删除优惠权益");
        log.setBizData(where);
        request.setAttribute("log", log);

		return ResponseBase.SUCCESS;
	}

	/**
	 * 修改优惠权益
	 *
	 * @param coupon 更新条件，不能为null
     * @param request HTTP请求
	 *
	 * @return 成功修改的记录数量
	 */
	@ApiIgnore
	@PostMapping("/update")
	public ResponseBase update(@RequestBody Coupon coupon, HttpServletRequest request) {
		// 为防止更新意外，必须传入id才能更新
		Long id = coupon.getId();

		if(id == null || id == 0L) {
			return ResponseBase.DATA_NOT_FOUND;
		}

		// 是否存在重复记录？
		if(exist(coupon)) {
			return ResponseBase.DATA_DUPLICATE;
		}

		// 第1个参数coupon，取关键字段coupon.id为条件
		// 第3个参数coupon，取coupon内关键字段以外其它属性数据
		int n = CouponService.update(coupon, null, coupon);

		if(n != 1) {
			return ResponseBase.DATA_NOT_FOUND;
		}

        // 记录日志
        OperationLog log = new OperationLog();
        log.setType("修改优惠权益");
        log.setBizData(coupon);
        request.setAttribute("log", log);

		// 重新查询，返回到前端
		coupon = CouponService.getObject(coupon, null);
		DataResponse<Coupon> result = new DataResponse<Coupon>();
		result.setData(coupon);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 查询一条优惠权益，并转化为相应的POJO对象
	 *
	 * @param coupon 查询条件，不能为null
	 *
	 * @return 返回结果记录，并转化为相应的POJO对象
	 */
	@ApiIgnore
	@PostMapping("/get")
	public ResponseBase get(@RequestBody Coupon coupon) {

		if(coupon.getId() == null || coupon.getId() == 0L) {
			return ResponseBase.DATA_NOT_FOUND;
		}

		// 查询
		coupon = CouponService.getObject(coupon, null);
		DataResponse<Coupon> result = new DataResponse<Coupon>();
		result.setData(coupon);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 查询多条优惠权益，并转化为相应的POJO对象列表
	 *
	 * @param coupon 更新条件，不能为null
	 *
	 * @return 返回结果记录，并转化为相应的POJO对象列表
	 */
	@ApiIgnore
	@PostMapping("/list")
	public ResponseBase list(@RequestBody Coupon coupon, @EntityParam Pagination pagination) {

		// 附加限定条件，这里可以修改附加限定条件
		String appendix = null;

		List<Coupon> couponList = CouponService.list(coupon, appendix, pagination);

		// 统计符合条件的结果记录数量
		int recordCount = CouponService.count(coupon, appendix);
		pagination.setRecordCount(recordCount);

		DataResponse<Map<String, Object>> result = new DataResponse<Map<String, Object>>();
		Map<String, Object> datas = new HashMap<String, Object>();
		datas.put("list", couponList);
		datas.put("pagination", pagination);

		result.setData(datas);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 获得符合条件的优惠权益数量
	 *
	 * @param coupon 查询条件，不能为null
	 *
	 * @return 返回记录数量
	 */
	@ApiIgnore
	@PostMapping("/count")
	public ResponseBase count(@RequestBody Coupon coupon) {
		// 附加限定条件，这里可以修改附加限定条件
		String appendix = null;

		int n = CouponService.count(coupon, appendix);
		DataResponse<Integer> result = new DataResponse<Integer>();

		result.setData(n);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 用在新增、修改时检查数据库中是否存在重复记录（请务必保留该方法）
	 *
	 * @param temp
	 *            将要修改的优惠权益POJO
	 * @return true：已经存在 false：不存在
	 */
	private boolean exist(Coupon temp) {
		// 检查修改的优惠权益是否有重复记录
		Coupon form = new Coupon();

		// 关键字限定条件

		String str = (temp.getId() == null ? null : "Id != " + temp.getId());

		// 其它信息限定条件

		// 礼遇编号
		form.setLoyaltyId(temp.getLoyaltyId());

		// 权益名称
		form.setRightName(temp.getRightName());

		// 有效期类型标识{1：按日期期间 2：按有效期}
		form.setActiveTypeFlag(temp.getActiveTypeFlag());

		// 有效期-开始时间
		form.setActiveStartTime(temp.getActiveStartTime());

		// 有效期-结束时间
		form.setActiveEndTime(temp.getActiveEndTime());

		// 有效期-月数{-1无限制}
		form.setActiveMonth(temp.getActiveMonth());

		// 核销类型{1：订单核销 2：快速核销}
		form.setUsedTypeFlag(temp.getUsedTypeFlag());

		// 是否可核销多次{1:是 2:否}
		form.setUseNumTypeFlag(temp.getUseNumTypeFlag());

		// 阶段性可核销次数
		form.setPhasedUseNum(temp.getPhasedUseNum());

		// 每{1:不限制 2:年 3:季度 4:月}
		form.setPhaseTypeFlag(temp.getPhaseTypeFlag());

		// 共可核销次数{-1不限制}
		form.setTotalUseNum(temp.getTotalUseNum());

		// 权益使用必须完成订单次数
		form.setUseLimitOrderNum(temp.getUseLimitOrderNum());

		// 权益类型{1:满减券金额类-整单减2:满减券金额类-每满减3:满减券产品类-整单减4:满减券产品类-每满减 5:折扣券6赠品券7免单券8代金券9动态折扣券}
		form.setRightTypeFlag(temp.getRightTypeFlag());

		// 权益内容
		form.setRightContent(temp.getRightContent());

		// 权益描述
		form.setDescription(temp.getDescription());

		// 使用说明
		form.setUseDescription(temp.getUseDescription());

		// 创建时间
		form.setCreateAt(temp.getCreateAt());

		// 更新时间
		form.setUpdateAt(temp.getUpdateAt());

		// 排序值
		form.setSortValue(temp.getSortValue());

		// 创建人编号
		form.setCreatorId(temp.getCreatorId());

		// 修改人编号
		form.setUpdatorId(temp.getUpdatorId());

		// 删除状态{0:未删除1:已删除}
		form.setDelStatus(temp.getDelStatus());

		// 有效状态{1:有效 2:暂停}
		form.setActiveStatus(temp.getActiveStatus());

		return (CouponService.getObject(form, str) != null);
	}

	/**
	 * 查询优惠券列表
	 *
	 *
	 * @return
	 */
	@PostMapping("/querycouponlist")
	@ApiOperation(value = "获取优惠券列表", notes = "优惠券列表", httpMethod = "POST")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "dms_code", value = "经销商code", dataType = "string"),
			@ApiImplicitParam(paramType = "query", name = "pageNo", value = "分页参数 - 页号，默认为1", dataType = "Integer"),
			@ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页参数 - 每页最大记录数量，默认为10", dataType = "Integer")
	})
	public ResponseBase list(String dms_code,Integer pageNo, Integer pageSize) {
		if(pageNo == null) pageNo = 1;
		if(pageSize == null) pageSize = 10;

		// 分页数据
		Pagination pagination = new Pagination();
		pagination.setRecordCount(Integer.MAX_VALUE);
		pagination.setMaxResults(pageSize);
		pagination.setPageNo(pageNo);

		// 统计符合条件的结果记录数量
		int recordCount = 0;
		pagination.setRecordCount(recordCount);

		List<Coupon> couponList = CouponService.queryCouponList(dms_code,pagination);

		DataResponse<Map<String, Object>> result = new DataResponse<Map<String, Object>>();
		Map<String, Object> datas = new HashMap<String, Object>();
		datas.put("list", couponList);
		datas.put("pagination", pagination);

		result.setData(datas);
		result.setMessage("成功");
		return result;
	}
}
